package com.tsf.demo.controller;

import com.tsf.demo.service.MiddlemanFeignService;
import com.tsf.demo.service.MiddlemanDubboServiceProxy;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.tsf.core.TsfContext;
import org.springframework.tsf.core.entity.Tag;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

@RestController
public class ConsumerController {
    private static final Logger LOG = LoggerFactory.getLogger(ConsumerController.class);

    @Autowired
    MiddlemanDubboServiceProxy middlemanServiceDubboProxy;

    @Autowired
    MiddlemanFeignService middlemanFeignService;

    // 1. 服务注册与发现
    // [http] 表示接收 http 协议并返回执行结果
    // [dubbo] 表示接收 dubbo 协议并返回执行结果
    // [http-http] 表示接收 http 协议，使用 http 协议调用下游
    // [http-dubbo] 表示接收 http 协议，使用 dubbo 协议调用下游
    // [dubbo-http] 表示接收 dubbo 协议，使用 http 协议调用下游
    // [dubbo-dubbo] 表示接收 dubbo 协议，使用 dubbo 协议调用下游
    // 测试示例如下：

    // [http-http] -> [http]
    @RequestMapping(value = "/http/echo/{param}", method = RequestMethod.GET)
    public String discoveryTestHttpEcho(@PathVariable String param) {
        return middlemanFeignService.httpEcho(param);
    }

    // [http-http] -> [http-dubbo] -> [dubbo]
    @RequestMapping(value = "/http/dubbo/echo/{param}", method = RequestMethod.GET)
    public String discoveryTestHttp2dubboEcho(@PathVariable String param) {
        return middlemanFeignService.dubboEcho(param);
    }

    // [http->dubbo] -> dubbo
    @RequestMapping(value = "/dubbo/echo/{param}", method = RequestMethod.GET)
    public String discoveryTestDubboEcho(@PathVariable String param) {
        return middlemanServiceDubboProxy.dubboEcho(param);
    }

    // [http-dubbo] -> [dubbo-http] -> [http]
    @RequestMapping(value = "/dubbo/http/echo/{param}", method = RequestMethod.GET)
    public String discoveryTestDubbo2HttpEcho(@PathVariable String param) {
        return middlemanServiceDubboProxy.dubbo2HttpEcho(param);
    }

    // [http-dubbo] -> [dubbo-dubbo] -> [dubbo]
    @RequestMapping(value = "/dubbo/dubbo/echo/{param}", method = RequestMethod.GET)
    public String discoveryTestDubbo2DubboEcho(@PathVariable String param) {
        return middlemanServiceDubboProxy.dubbo2DubboEcho(param);
    }

    // 2. 动态配置
    @RequestMapping(value = "/config/test", method = RequestMethod.GET)
    public String configTest() {
        return middlemanFeignService.configTest();
    }

    // 3. 鉴权限流路由熔断

    @RequestMapping(value = "/auth/tag/dubbo/{tagName}/{tagValue}", method = RequestMethod.GET)
    public String tagTestDubbo(@PathVariable String tagName, @PathVariable String tagValue) {
        TsfContext.putTag(tagName, tagValue, Tag.ControlFlag.TRANSITIVE);
        return middlemanServiceDubboProxy.dubboEcho(String.format("%s-%s", tagName, tagValue));
    }

    @RequestMapping(value = "/auth/tag/http/{tagName}/{tagValue}", method = RequestMethod.GET)
    public String tagTestHttp(@PathVariable String tagName, @PathVariable String tagValue) {
        TsfContext.putTag(tagName, tagValue, Tag.ControlFlag.TRANSITIVE);
        return middlemanFeignService.httpEcho(String.format("%s-%s", tagName, tagValue));
    }
}