package com.tsf.demo.controller;

import com.tsf.demo.dubbo.service.ProviderDubboService;
import org.apache.dubbo.config.annotation.DubboReference;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;

import java.io.IOException;

import static com.tsf.demo.controller.CommonUtils.getInet4Address;

@RestController
public class MiddlemanController {
    private static final Logger LOG = LoggerFactory.getLogger(MiddlemanController.class);

    @Autowired
    private MiddlemanNameConfig middlemanNameConfig;

    @DubboReference(timeout = 5000, check = false, version = "2.0")
    private ProviderDubboService providerDubboService;

    @RequestMapping(value = "/http/echo/{param}", method = RequestMethod.GET)
    public String httpEcho(@PathVariable String param, HttpServletResponse response) throws IOException {
        switch (param) {
            case "1xx":
                response.setStatus(HttpServletResponse.SC_CONTINUE);
                response.getWriter().write("mock 1xx return.");
                response.getWriter().flush();
                return "mock 1xx return.";
            case "3xx":
                response.setStatus(HttpServletResponse.SC_FOUND);
                response.getWriter().write("mock 3xx return.");
                response.getWriter().flush();
                return "mock 3xx return.";
            case "4xx":
                response.setStatus(HttpServletResponse.SC_NOT_FOUND);
                response.getWriter().write("mock 4xx return.");
                response.getWriter().flush();
                return "mock 4xx return.";
            case "5xx":
                response.setStatus(HttpServletResponse.SC_BAD_GATEWAY);
                response.getWriter().write("mock 5xx return.");
                response.getWriter().flush();
                return "mock 5xx return.";
            default:
                LOG.info("middleman-http -- request param: [" + param + "]");
                String result = "middleman received http, from host-ip: " + getInet4Address() + ",request param: " + param;
                LOG.info("middleman-http -- response info: [" + result + "]");
                return result;
        }
    }

    @RequestMapping(value = "/dubbo/echo/{param}", method = RequestMethod.GET)
    public String dubboEcho(@PathVariable("param") String param) {
        LOG.info("middleman-http -- request param: [" + param + "]");
        String result = "middleman received http, " + providerDubboService.echo(param);
        LOG.info("middleman-http -- response info: [" + result + "]");
        return result;
    }

    @RequestMapping(value = "/config/test", method = RequestMethod.GET)
    public String configTest() {
        return "middleman test dynamic config: " + middlemanNameConfig.getName();
    }
}